#!/usr/bin/env python
#coding:utf-8
"""
  Author:   --<v1ll4n>
  Purpose: Test for decologger
  Created: 2016/12/18
"""

import unittest
import traceback
from .decologger import Decologger


########################################################################
class DecologgerTest(unittest.case.TestCase):
    """"""

    #----------------------------------------------------------------------
    def test_learn_decorector(self):
        """"""
        print(('='*64))
        def logit(func):
            def wrapper(*args, **kwargs):
                print(('wrapper got param', args, kwargs))
                print(('pre call', func.__name__))
                return func(*args, **kwargs)
            return wrapper
    
        #----------------------------------------------------------------------
        @logit
        def funct(args):
            """"""
            print(('funct called! Got Param', args))
        
        funct('Hello')
        
        def sss(*arg):
            print(arg)
            print('sssss called') 
            return logit(arg[0])
        
        @sss
        def tests():
            print('tests called')
        
        tests()
        
        print(('='*64))
    
    def test_exception(self):
        try:
            raise Exception('adfasdfasdf')
        except:
            print(traceback.extract_stack())
            traceback.format_exc()
    
    #----------------------------------------------------------------------
    def test_decologger_basic_api(self):
        """"""
        dclogger = Decologger(name='testlogger')
        
        
        class A(object):
            #----------------------------------------------------------------------
            def __init__(self):
                """"""
                pass
            
            #----------------------------------------------------------------------
            @dclogger.crucial
            def B(self):
                """"""
                print('B Called!')
        
        
        dclogger.critical('Hello Critical')
        A().B()
                
        
        print(('='*64))
        
        
        
    
    

if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python
#coding:utf-8
"""
  Author:   --<v1ll4n>
  Purpose: Queue Utils
  Created: 2017/1/3
"""

import unittest
import threading
import time
from queue import Empty

#----------------------------------------------------------------------
def sync_dispatch(queue_instance, task_generator, 
                  buffer_size=100, interval=0.05):
    """To dispatch tasks from [task_generator] to [queue_instance] 
    with the [buffer_size] buffer task.
    
    Params:
        queue_instance: :Queue: receive task.
        task_generator: :generator/list/tuple/set: Who have tasks 
            waiting for being dispatched.
        buffer_size: :int: buffer_size.
        interval: :float/int: interval of checking the size of queue."""
    
    assert isinstance(interval, (int, float)), \
           '[!] [interval] should be a float/int! '
    assert isinstance(buffer_size, int), \
           '[!] [buffer_size] should be a int! '
    
    for i in task_generator:
        while queue_instance.qsize() >= buffer_size:
            time.sleep(interval)
        queue_instance.put(i)

#----------------------------------------------------------------------
def async_dispatch(queue_instance, task_generator, 
                  buffer_size=100, interval=0.05,
                  thread_name='ASYNC_DISPATCHER', daemon=True):
    """To dispatch tasks from [task_generator] to [queue_instance] 
    with the [buffer_size] buffer task.
    
    Params:
        queue_instance: :Queue: receive task.
        task_generator: :generator/list/tuple/set: Who have tasks 
            waiting for being dispatched.
        buffer_size: :int: buffer_size
        interval: :float/int: interval of checking the size of queue.
        thread_name: :str: the name of thread.
        daemon: :bool: daemon thread?"""
    
    assert isinstance(interval, (int, float)), \
           '[!] [interval] should be a float/int! '
    assert isinstance(buffer_size, int), \
           '[!] [buffer_size] should be a int! '
    
    def _worker():
        for i in task_generator:
            while queue_instance.qsize() >= buffer_size:
                time.sleep(interval)
            queue_instance.put(i)
    
    _threadi = threading.Thread(name=thread_name, target=_worker)
    _threadi.daemon = True
    _threadi.start()
    
#----------------------------------------------------------------------
def queue_peek(queue_instance, timeout=60):
    """Convert Queue to A generator with a timeout
    
    Params:
        queue_instance: :Queue: the Queue you want to peek
        timeout: :int/float: from got a """
    while True:
        try:
            yield queue_instance.get(timeout=timeout)
        except Empty:
            break
    

if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python
#coding:utf-8
"""
  Author:   --<v1ll4n>
  Purpose: TestForDictParser
  Created: 2016/12/16
"""

import unittest
from .dict_parser import *


########################################################################
class DictParserTest(unittest.case.TestCase):
    """"""

    #----------------------------------------------------------------------
    def test_dict_basic_api(self):
        """Constructor"""
        pprint('='*64)
        pprint('Test Dict Parser Parse A Small Dictionary')
        dictparse = DictParser(filename='dir.txt', do_continue=False)
        
        count = 0
        for i in dictparse:
            pprint(i)
            count = count + 1
            if count > 10:
                break
            
        dictparse = DictParser(filename='dir.txt', do_continue=True)
        retcollect = dictparse.get_next_collection(num=200)
        
        for i in retcollect:
            pprint(i)
        pprint('='*64)
    
    #----------------------------------------------------------------------
    def test_get_an_collection(self):
        """"""
        
        pprint('='*64)
        pprint('Test get a collection by dictparser')
        dictparse = DictParser(filename='dir.txt', do_continue=False)
        retcollect = dictparse.get_next_collection(num=200)
        
        for i in retcollect:
            pprint(i)
        pprint('='*64)
        
    #----------------------------------------------------------------------
    def test_get_basic_information(self):
        """"""
        pprint('='*64)
        pprint('Test get basic information of dictfile')
        dictparse = DictParser(filename='dir.txt', do_continue=False)
        pprint("Current Pos: %d" % dictparse.get_current_pos())
        pprint("Totol SIZE: %d" % dictparse.get_total_size())
        dictparse.get_fp().readline()
        pprint("Current Pos: %d" % dictparse.get_current_pos())
        pprint("Totol SIZE: %d" % dictparse.get_total_size())  
        #dictparse.reset()
        dictparse.get_fp().readline()
        pprint("Current Pos: %d" % dictparse.get_current_pos())
        pprint("Totol SIZE: %d" % dictparse.get_total_size())        
        pprint('='*64)
        

if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python
#coding:utf-8
"""
  Author:   --<v1ll4n>
  Purpose: Make inspect easier
  Created: 2017/1/2
"""

import unittest
import inspect
from types import FunctionType
from types import MethodType
from pprint import pprint
#import ip_calc_utils 

#----------------------------------------------------------------------
def get_callables(module_or_instance):
    """Get all callable mamber from [module_or_instance]
    
    Params:
        module_or_instance: :str: the name of module or instance
    
    Returns:
        A list stored the callable objects"""
    result = []
    
    for i in inspect.getmembers(module_or_instance):
        if callable(i[1]):
            result.append(i[1])
        else:
            pass
        
    return result

#----------------------------------------------------------------------
def get_fileds(module_or_instance, public=True):
    """Get all field member from [module_or_instance]
    
    Params:
        module_or_instance: :str: the name of module or instance
         
    Returns:
        A dict stored the keys and values."""
    result = {}
    
    for i in inspect.getmembers(module_or_instance):
        if not callable(i[1]):
            if i[0].startswith('_'):
                pass
            else:
                result[i[0]] = i[1]
        else:
            pass
        
    return result

#----------------------------------------------------------------------
def get_functions(module_or_instance, public=True):
    """Get all function from [module_or_instance]
    
    Params:
        module_or_instance: :str: the name of module or instances
        public: :bool: get the public function?
            'public' means that the [name].startswith('_') is true.
    
    Returns:
        A list stored the function from module/instance
    """
    ret = []
    
    for i in get_callables(module_or_instance):
        if isinstance(i, FunctionType):
            ret.append(i)
    
    if public:
        ret = [x for x in ret if None if x.__name__.startswith('_') else x]
    else:
        pass
    
    while True:
        try:
            ret.remove(None)
        except ValueError:
            break
        
    return ret

#----------------------------------------------------------------------
def get_methods(instance, public=True):
    """Get methods from instance"""
    
    ret = []
    
    for i in get_callables(instance):
        if isinstance(i, MethodType):
            ret.append(i)
    
    if public:
        ret = [x for x in ret if None if x.__name__.startswith('_') \
                     else x]
    else:
        pass
    
    while True:
        try:
            ret.remove(None)
        except ValueError:
            break
        
    return ret    

#----------------------------------------------------------------------
def get_args_dict(func):
    """"""
    assert callable(func), '[!] The [func] you input cannot be called!'
    
    result = {}
    
    try:
        func_name = getattr(func, 'func_name')
    except:
        func_name = str(func)
    
    #
    # Process args
    #
    args = {}
    spec = inspect.getargspec(func)
    args['vargs'] = spec.varargs
    args['kwargs'] = spec.keywords
    args['defaults'] = spec.defaults if spec.defaults else []
    args['args'] = spec.args
    args['args_table'] = {}
    
    for i in spec.args:
        args['args_table'][i] = None
    
    for i in range(len(args['defaults'])):
        index = -1 - i
        args['args_table'][spec.args[index]] = spec.defaults[index]
    
    result['func_name'] = func_name
    result['args'] = args
    
    return result

#----------------------------------------------------------------------
def get_neccessary_params(func):
    """"""
    assert callable(func), '[!] The [func] you input cannot be called!'
    
    result = []
    
    table = get_args_dict(func)['args']['args_table']
    for i in list(table.items()):
        if i[1] != None:
            pass
        else:
            result.append(i[0])
            
    return result
    


if __name__ == '__main__':
    unittest.main()


# Copyright (c) 2013, 2014, 2015, 2016 Philip Hane
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import sys
import socket
import dns.resolver
import json
import logging
from time import sleep

# Import the dnspython3 rdtypes to fix the dynamic import problem when frozen.
import dns.rdtypes.ANY.TXT  # @UnusedImport

from .exceptions import (IPDefinedError, ASNRegistryError, ASNLookupError,
                         BlacklistError, WhoisLookupError, HTTPLookupError,
                         HostLookupError, HTTPRateLimitError)
from .whois import RIR_WHOIS
from .utils import ipv4_is_defined, ipv6_is_defined

if sys.version_info >= (3, 3):  # pragma: no cover
    from ipaddress import (ip_address,
                           IPv4Address,
                           IPv6Address,
                           ip_network,
                           summarize_address_range,
                           collapse_addresses)
else:  # pragma: no cover
    from ipaddr import (IPAddress as ip_address,
                        IPv4Address,
                        IPv6Address,
                        IPNetwork as ip_network,
                        summarize_address_range,
                        collapse_address_list as collapse_addresses)

try:  # pragma: no cover
    from urllib.request import (OpenerDirector,
                                ProxyHandler,
                                build_opener,
                                Request,
                                URLError)
    from urllib.parse import urlencode
except ImportError:  # pragma: no cover
    from urllib2 import (OpenerDirector,
                         ProxyHandler,
                         build_opener,
                         Request,
                         URLError)
    from urllib.parse import urlencode

log = logging.getLogger(__name__)

# POSSIBLY UPDATE TO USE RDAP
ARIN = 'http://whois.arin.net/rest/nets;q={0}?showDetails=true&showARIN=true'

CYMRU_WHOIS = 'whois.cymru.com'

IPV4_DNS_ZONE = '{0}.origin.asn.cymru.com'

IPV6_DNS_ZONE = '{0}.origin6.asn.cymru.com'

BLACKLIST = [
    'root.rwhois.net'
]

ORG_MAP = {
    'ARIN': 'arin',
    'VR-ARIN': 'arin',
    'RIPE': 'ripencc',
    'APNIC': 'apnic',
    'LACNIC': 'lacnic',
    'AFRINIC': 'afrinic',
    'DNIC': 'arin'
}


class Net:
    """
    The class for performing network queries.

    Args:
        address: An IPv4 or IPv6 address in string format.
        timeout: The default timeout for socket connections in seconds.
        proxy_opener: The urllib.request.OpenerDirector request for proxy
            support or None.
        allow_permutations: Use additional methods if DNS lookups to Cymru
            fail.

    Raises:
        IPDefinedError: The address provided is defined (does not need to be
            resolved).
    """

    def __init__(self, address, timeout=5, proxy_opener=None,
                 allow_permutations=True):

        # IPv4Address or IPv6Address
        if isinstance(address, IPv4Address) or isinstance(
                address, IPv6Address):

            self.address = address

        else:

            # Use ipaddress package exception handling.
            self.address = ip_address(address)

        # Default timeout for socket connections.
        self.timeout = timeout

        # Allow other than DNS lookups for ASNs.
        self.allow_permutations = allow_permutations

        self.dns_resolver = dns.resolver.Resolver()
        self.dns_resolver.timeout = timeout
        self.dns_resolver.lifetime = timeout

        # Proxy opener.
        if isinstance(proxy_opener, OpenerDirector):

            self.opener = proxy_opener

        else:

            handler = ProxyHandler()
            self.opener = build_opener(handler)

        # IP address in string format for use in queries.
        self.address_str = self.address.__str__()

        # Determine the IP version, 4 or 6
        self.version = self.address.version

        if self.version == 4:

            # Check if no ASN/whois resolution needs to occur.
            is_defined = ipv4_is_defined(self.address_str)

            if is_defined[0]:

                raise IPDefinedError(
                    'IPv4 address {0} is already defined as {1} via '
                    '{2}.'.format(
                        self.address_str, is_defined[1], is_defined[2]
                    )
                )

            # Reverse the IPv4Address for the DNS ASN query.
            split = self.address_str.split('.')
            split.reverse()
            self.reversed = '.'.join(split)

            self.dns_zone = IPV4_DNS_ZONE.format(self.reversed)

        else:

            # Check if no ASN/whois resolution needs to occur.
            is_defined = ipv6_is_defined(self.address_str)

            if is_defined[0]:

                raise IPDefinedError(
                    'IPv6 address {0} is already defined as {1} via '
                    '{2}.'.format(
                        self.address_str, is_defined[1], is_defined[2]
                    )
                )

            # Explode the IPv6Address to fill in any missing 0's.
            exploded = self.address.exploded

            # Cymru seems to timeout when the IPv6 address has trailing '0000'
            # groups. Remove these groups.
            groups = exploded.split(':')
            for index, value in reversed(list(enumerate(groups))):

                if value == '0000':

                    del groups[index]

                else:

                    break

            exploded = ':'.join(groups)

            # Reverse the IPv6Address for the DNS ASN query.
            val = str(exploded).replace(':', '')
            val = val[::-1]
            self.reversed = '.'.join(val)

            self.dns_zone = IPV6_DNS_ZONE.format(self.reversed)

    def get_asn_dns(self, result=None):
        """
        The function for retrieving ASN information for an IP address from
        Cymru via port 53 (DNS).

        Args:
            result: Optional result object. This bypasses the ASN lookup.

        Returns:
            Dictionary: A dictionary containing the following keys:
                    asn (String) - The Autonomous System Number.
                    asn_date (String) - The ASN Allocation date.
                    asn_registry (String) - The assigned ASN registry.
                    asn_cidr (String) - The assigned ASN CIDR.
                    asn_country_code (String) - The assigned ASN country code.

        Raises:
            ASNRegistryError: The ASN registry is not known.
            ASNLookupError: The ASN lookup failed.
        """

        try:

            if result is None:

                log.debug('ASN query for {0}'.format(self.dns_zone))
                data = self.dns_resolver.query(self.dns_zone, 'TXT')
                temp = str(data[0]).split('|')

            else:

                temp = result

            # Parse out the ASN information.
            ret = {'asn_registry': temp[3].strip(' \n')}

            if ret['asn_registry'] not in list(RIR_WHOIS.keys()):

                raise ASNRegistryError(
                    'ASN registry {0} is not known.'.format(
                        ret['asn_registry'])
                )

            ret['asn'] = temp[0].strip(' "\n')
            ret['asn_cidr'] = temp[1].strip(' \n')
            ret['asn_country_code'] = temp[2].strip(' \n').upper()
            ret['asn_date'] = temp[4].strip(' "\n')

            return ret

        except ASNRegistryError:

            raise

        except (dns.resolver.NXDOMAIN, dns.resolver.NoNameservers,
                dns.resolver.NoAnswer, dns.exception.Timeout) as e:

            raise ASNLookupError(
                'ASN lookup failed (DNS {0}) for {1}.'.format(
                    e.__class__.__name__, self.address_str)
            )

        except:

            raise ASNLookupError(
                'ASN lookup failed for {0}.'.format(self.address_str)
            )

    def get_asn_whois(self, retry_count=3, result=None):
        """
        The function for retrieving ASN information for an IP address from
        Cymru via port 43/tcp (WHOIS).

        Args:
            retry_count: The number of times to retry in case socket errors,
                timeouts, connection resets, etc. are encountered.
            result: Optional result object. This bypasses the ASN lookup.

        Returns:
            Dictionary: A dictionary containing the following keys:
                    asn (String) - The Autonomous System Number.
                    asn_date (String) - The ASN Allocation date.
                    asn_registry (String) - The assigned ASN registry.
                    asn_cidr (String) - The assigned ASN CIDR.
                    asn_country_code (String) - The assigned ASN country code.

        Raises:
            ASNRegistryError: The ASN registry is not known.
            ASNLookupError: The ASN lookup failed.
        """

        try:

            if result is None:

                # Create the connection for the Cymru whois query.
                conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                conn.settimeout(self.timeout)
                log.debug('ASN query for {0}'.format(self.address_str))
                conn.connect((CYMRU_WHOIS, 43))

                # Query the Cymru whois server, and store the results.
                conn.send((
                    ' -r -a -c -p -f -o {0}{1}'.format(
                        self.address_str, '\r\n')
                ).encode())

                data = ''
                while True:

                    d = conn.recv(4096).decode()
                    data += d

                    if not d:

                        break

                conn.close()

            else:

                data = result

            # Parse out the ASN information.
            temp = str(data).split('|')

            ret = {'asn_registry': temp[4].strip(' \n')}

            if ret['asn_registry'] not in list(RIR_WHOIS.keys()):

                raise ASNRegistryError(
                    'ASN registry {0} is not known.'.format(
                        ret['asn_registry'])
                )

            ret['asn'] = temp[0].strip(' \n')
            ret['asn_cidr'] = temp[2].strip(' \n')
            ret['asn_country_code'] = temp[3].strip(' \n').upper()
            ret['asn_date'] = temp[5].strip(' \n')

            return ret

        except (socket.timeout, socket.error) as e:  # pragma: no cover

            log.debug('ASN query socket error: {0}'.format(e))
            if retry_count > 0:

                log.debug('ASN query retrying (count: {0})'.format(
                    str(retry_count)))
                return self.get_asn_whois(retry_count - 1)

            else:

                raise ASNLookupError(
                    'ASN lookup failed for {0}.'.format(self.address_str)
                )

        except ASNRegistryError:

            raise

        except:

            raise ASNLookupError(
                'ASN lookup failed for {0}.'.format(self.address_str)
            )

    def get_asn_http(self, retry_count=3, result=None, extra_org_map=None):
        """
        The function for retrieving ASN information for an IP address from
        Arin via port 80 (HTTP). Currently limited to fetching asn_registry
        through a Arin whois (REST) lookup. The other values are returned as
        None to keep a consistent dict output. This should be used as a last
        chance fallback call behind ASN DNS & ASN Whois lookups.

        Args:
            retry_count: The number of times to retry in case socket errors,
                timeouts, connection resets, etc. are encountered.
            result: Optional result object. This bypasses the ASN lookup.
            extra_org_map: Dictionary mapping org handles to RIRs. This is for
                limited cases where ARIN REST (ASN fallback HTTP lookup) does
                not show an RIR as the org handle e.g., DNIC (which is now the
                built in ORG_MAP) e.g., {'DNIC': 'arin'}. Valid RIR values are
                (note the case-sensitive - this is meant to match the REST
                result): 'ARIN', 'RIPE', 'apnic', 'lacnic', 'afrinic'

        Returns:
            Dictionary: A dictionary containing the following keys:
                    asn (String) - None, can't retrieve with this method.
                    asn_date (String) - None, can't retrieve with this method.
                    asn_registry (String) - The assigned ASN registry.
                    asn_cidr (String) - None, can't retrieve with this method.
                    asn_country_code (String) - None, can't retrieve with this
                    method.

        Raises:
            ASNRegistryError: The ASN registry is not known.
            ASNLookupError: The ASN lookup failed.
        """

        # Set the org_map. Map the orgRef handle to an RIR.
        org_map = ORG_MAP.copy()
        try:

            org_map.update(extra_org_map)

        except (TypeError, ValueError, IndexError, KeyError):

            pass

        try:

            if result is None:

                # Lets attempt to get the ASN registry information from
                # ARIN.
                log.debug('ASN query for {0}'.format(self.address_str))
                response = self.get_http_json(
                    url=str(ARIN).format(self.address_str),
                    retry_count=retry_count,
                    headers={'Accept': 'application/json'}
                )

            else:

                response = result

            asn_data = {
                'asn_registry': None,
                'asn': None,
                'asn_cidr': None,
                'asn_country_code': None,
                'asn_date': None
            }

            try:

                net_list = response['nets']['net']

                if not isinstance(net_list, list):
                    net_list = [net_list]

            except (KeyError, TypeError):

                log.debug('No networks found')
                net_list = []

            for n in net_list:

                try:

                    asn_data['asn_registry'] = (
                        org_map[n['orgRef']['@handle'].upper()]
                    )

                except KeyError as e:

                    log.debug('Could not parse ASN registry via HTTP: '
                              '{0}'.format(str(e)))
                    raise ASNRegistryError('ASN registry lookup failed.')

                break

            return asn_data

        except (socket.timeout, socket.error) as e:  # pragma: no cover

            log.debug('ASN query socket error: {0}'.format(e))
            if retry_count > 0:

                log.debug('ASN query retrying (count: {0})'.format(
                    str(retry_count)))
                return self.get_asn_http(retry_count=retry_count-1)

            else:

                raise ASNLookupError(
                    'ASN lookup failed for {0}.'.format(self.address_str)
                )

        except ASNRegistryError:

            raise

        except:

            raise ASNLookupError(
                'ASN lookup failed for {0}.'.format(self.address_str)
            )

    def get_whois(self, asn_registry='arin', retry_count=3, server=None,
                  port=43, extra_blacklist=None):
        """
        The function for retrieving whois or rwhois information for an IP
        address via any port. Defaults to port 43/tcp (WHOIS).

        Args:
            asn_registry: The NIC to run the query against.
            retry_count: The number of times to retry in case socket errors,
                timeouts, connection resets, etc. are encountered.
            server: An optional server to connect to. If provided, asn_registry
                will be ignored.
            port: The network port to connect on.
            extra_blacklist: A list of blacklisted whois servers in addition to
                the global BLACKLIST.

        Returns:
            String: The raw whois data.

        Raises:
            BlacklistError: Raised if the whois server provided is in the
                global BLACKLIST or extra_blacklist.
            WhoisLookupError: The whois lookup failed.
        """

        try:

            extra_bl = extra_blacklist if extra_blacklist else []

            if any(server in srv for srv in (BLACKLIST, extra_bl)):
                raise BlacklistError(
                    'The server {0} is blacklisted.'.format(server)
                )

            if server is None:
                server = RIR_WHOIS[asn_registry]['server']

            # Create the connection for the whois query.
            conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            conn.settimeout(self.timeout)
            log.debug('WHOIS query for {0} at {1}:{2}'.format(
                self.address_str, server, port))
            conn.connect((server, port))

            # Prep the query.
            query = self.address_str + '\r\n'
            if asn_registry == 'arin':

                query = 'n + {0}'.format(query)

            # Query the whois server, and store the results.
            conn.send(query.encode())

            response = ''
            while True:

                d = conn.recv(4096).decode('ascii', 'ignore')

                response += d

                if not d:

                    break

            conn.close()

            if 'Query rate limit exceeded' in response:  # pragma: no cover

                log.debug('WHOIS query rate limit exceeded. Waiting...')
                sleep(1)
                return self.get_whois(
                    asn_registry=asn_registry, retry_count=retry_count-1,
                    server=server, port=port, extra_blacklist=extra_blacklist
                )

            elif ('error 501' in response or 'error 230' in response
                  ):  # pragma: no cover

                log.debug('WHOIS query error: {0}'.format(response))
                raise ValueError

            return str(response)

        except (socket.timeout, socket.error) as e:

            log.debug('WHOIS query socket error: {0}'.format(e))
            if retry_count > 0:

                log.debug('WHOIS query retrying (count: {0})'.format(
                    str(retry_count)))
                return self.get_whois(
                    asn_registry=asn_registry, retry_count=retry_count-1,
                    server=server, port=port, extra_blacklist=extra_blacklist
                )

            else:

                raise WhoisLookupError(
                    'WHOIS lookup failed for {0}.'.format(self.address_str)
                )

        except BlacklistError:

            raise

        except:  # pragma: no cover

            raise WhoisLookupError(
                'WHOIS lookup failed for {0}.'.format(self.address_str)
            )

    def get_http_json(self, url=None, retry_count=3, rate_limit_timeout=120,
                      headers=None):
        """
        The function for retrieving a json result via HTTP.

        Args:
            url: The URL to retrieve.
            retry_count: The number of times to retry in case socket errors,
                timeouts, connection resets, etc. are encountered.
            rate_limit_timeout: The number of seconds to wait before retrying
                when a rate limit notice is returned via rdap+json.
            headers: The HTTP headers dictionary. The Accept header defaults
                to 'application/rdap+json'.

        Returns:
            Dictionary: The data in json format.

        Raises:
            HTTPLookupError: The HTTP lookup failed.
            HTTPRateLimitError: The HTTP request rate limited and retries
                were exhausted.
        """

        if headers is None:
            headers = {'Accept': 'application/rdap+json'}

        try:

            # Create the connection for the whois query.
            log.debug('HTTP query for {0} at {1}'.format(
                self.address_str, url))
            conn = Request(url, headers=headers)
            data = self.opener.open(conn, timeout=self.timeout)
            try:
                d = json.loads(data.readall().decode('utf-8', 'ignore'))
            except AttributeError:  # pragma: no cover
                d = json.loads(data.read().decode('utf-8', 'ignore'))

            try:
                # Tests written but commented out. I do not want to send a
                # flood of requests on every test.
                for tmp in d['notices']:  # pragma: no cover
                    if tmp['title'] == 'Rate Limit Notice':
                        log.debug('RDAP query rate limit exceeded.')

                        if retry_count > 0:
                            log.debug('Waiting {0} seconds...'.format(
                                str(rate_limit_timeout)))

                            sleep(rate_limit_timeout)
                            return self.get_http_json(
                                url=url, retry_count=retry_count-1,
                                rate_limit_timeout=rate_limit_timeout,
                                headers=headers
                            )
                        else:
                            raise HTTPRateLimitError(
                                'HTTP lookup failed for {0}. Rate limit '
                                'exceeded, wait and try again (possibly a '
                                'temporary block).'.format(url))

            except (KeyError, IndexError):  # pragma: no cover

                pass

            return d

        except (URLError, socket.timeout, socket.error) as e:

            # Check needed for Python 2.6, also why URLError is caught.
            try:  # pragma: no cover
                if not isinstance(e.reason, (socket.timeout, socket.error)):
                    raise HTTPLookupError('HTTP lookup failed for {0}.'
                                          ''.format(url))
            except AttributeError:  # pragma: no cover

                pass

            log.debug('HTTP query socket error: {0}'.format(e))
            if retry_count > 0:

                log.debug('HTTP query retrying (count: {0})'.format(
                    str(retry_count)))

                return self.get_http_json(
                    url=url, retry_count=retry_count-1,
                    rate_limit_timeout=rate_limit_timeout, headers=headers
                )

            else:

                raise HTTPLookupError('HTTP lookup failed for {0}.'.format(
                    url))

        except (HTTPLookupError, HTTPRateLimitError) as e:  # pragma: no cover

            raise e

        except:  # pragma: no cover

            raise HTTPLookupError('HTTP lookup failed for {0}.'.format(url))

    def get_host(self, retry_count=3):
        """
        The function for retrieving host information for an IP address.

        Args:
            retry_count: The number of times to retry in case socket errors,
                timeouts, connection resets, etc. are encountered.

        Returns:
            Tuple: hostname, aliaslist, ipaddrlist

        Raises:
            HostLookupError: The host lookup failed.
        """

        try:

            default_timeout_set = False
            if not socket.getdefaulttimeout():

                socket.setdefaulttimeout(self.timeout)
                default_timeout_set = True

            log.debug('Host query for {0}'.format(self.address_str))
            ret = socket.gethostbyaddr(self.address_str)

            if default_timeout_set:  # pragma: no cover

                socket.setdefaulttimeout(None)

            return ret

        except (socket.timeout, socket.error) as e:

            log.debug('Host query socket error: {0}'.format(e))
            if retry_count > 0:

                log.debug('Host query retrying (count: {0})'.format(
                    str(retry_count)))

                return self.get_host(retry_count - 1)

            else:

                raise HostLookupError(
                    'Host lookup failed for {0}.'.format(self.address_str)
                )

        except:  # pragma: no cover

            raise HostLookupError(
                'Host lookup failed for {0}.'.format(self.address_str)
            )

    def lookup_asn(self, retry_count=3, asn_alts=None, extra_org_map=None):
        """
        The wrapper function for retrieving and parsing ASN information for an
        IP address.

        Args:
            retry_count: The number of times to retry in case socket errors,
                timeouts, connection resets, etc. are encountered.
            asn_alts: Array of additional lookup types to attempt if the
                ASN dns lookup fails. Allow permutations must be enabled.
                Defaults to all ['whois', 'http'].
            extra_org_map: Dictionary mapping org handles to RIRs. This is for
                limited cases where ARIN REST (ASN fallback HTTP lookup) does
                not show an RIR as the org handle e.g., DNIC (which is now the
                built in ORG_MAP) e.g., {'DNIC': 'arin'}. Valid RIR values are
                (note the case-sensitive - this is meant to match the REST
                result): 'ARIN', 'RIPE', 'apnic', 'lacnic', 'afrinic'

        Returns:
            Tuple:

            :Dictionary: Result from get_asn_dns() or get_asn_whois().
            :Dictionary: The response returned by get_asn_dns() or
                get_asn_whois().

        Raises:
            ASNRegistryError: ASN registry does not match.
            HTTPLookupError: The HTTP lookup failed.
        """

        lookups = asn_alts if asn_alts is not None else ['whois', 'http']

        # Initialize the response.
        response = None

        # Attempt to resolve ASN info via Cymru. DNS is faster, try that first.
        try:

            self.dns_resolver.lifetime = self.dns_resolver.timeout * (
                retry_count and retry_count or 1)
            asn_data = self.get_asn_dns()

        except (ASNLookupError, ASNRegistryError) as e:

            if not self.allow_permutations:

                raise ASNRegistryError('ASN registry lookup failed. '
                                       'Permutations not allowed.')

            try:
                if 'whois' in lookups:

                    log.debug('ASN DNS lookup failed, trying ASN WHOIS: '
                              '{0}'.format(e))
                    asn_data = self.get_asn_whois(retry_count)

                else:

                    raise ASNLookupError

            except (ASNLookupError, ASNRegistryError):  # pragma: no cover

                if 'http' in lookups:

                    # Lets attempt to get the ASN registry information from
                    # ARIN.
                    log.debug('ASN WHOIS lookup failed, trying ASN via HTTP')
                    try:

                        asn_data = self.get_asn_http(
                            retry_count=retry_count,
                            extra_org_map=extra_org_map
                        )

                    except ASNRegistryError:

                        raise ASNRegistryError('ASN registry lookup failed.')

                    except ASNLookupError:

                        raise HTTPLookupError('ASN HTTP lookup failed.')

                else:

                    raise ASNRegistryError('ASN registry lookup failed.')

        return asn_data, response

    def get_http_raw(self, url=None, retry_count=3, headers=None,
                     request_type='GET', form_data=None):
        """
        The function for retrieving a raw HTML result via HTTP.

        Args:
            url: The URL to retrieve.
            retry_count: The number of times to retry in case socket errors,
                timeouts, connection resets, etc. are encountered.
            headers: The HTTP headers dictionary. The Accept header defaults
                to 'application/rdap+json'.
            request_type: 'GET' or 'POST'
            form_data: Dictionary of form POST data

        Returns:
            String: The raw data.

        Raises:
            HTTPLookupError: The HTTP lookup failed.
        """

        if headers is None:
            headers = {'Accept': 'text/html'}

        if form_data:
            form_data = urlencode(form_data)
            try:
                # Py 2 inspection will alert on the encoding arg, no harm done.
                form_data = bytes(form_data, encoding='ascii')
            except TypeError:  # pragma: no cover
                pass

        try:

            # Create the connection for the HTTP query.
            log.debug('HTTP query for {0} at {1}'.format(
                self.address_str, url))
            try:
                # Py 2 inspection alert bypassed by using kwargs dict.
                conn = Request(url=url, data=form_data, headers=headers,
                               **{'method': request_type})
            except TypeError:  # pragma: no cover
                conn = Request(url=url, data=form_data, headers=headers)
            data = self.opener.open(conn, timeout=self.timeout)

            try:
                d = data.readall().decode('ascii', 'ignore')
            except AttributeError:  # pragma: no cover
                d = data.read().decode('ascii', 'ignore')

            return str(d)

        except (URLError, socket.timeout, socket.error) as e:

            # Check needed for Python 2.6, also why URLError is caught.
            try:  # pragma: no cover
                if not isinstance(e.reason, (socket.timeout, socket.error)):
                    raise HTTPLookupError('HTTP lookup failed for {0}.'
                                          ''.format(url))
            except AttributeError:  # pragma: no cover

                pass

            log.debug('HTTP query socket error: {0}'.format(e))
            if retry_count > 0:

                log.debug('HTTP query retrying (count: {0})'.format(
                    str(retry_count)))

                return self.get_http_raw(
                    url=url, retry_count=retry_count - 1, headers=headers,
                    request_type=request_type, form_data=form_data
                )

            else:

                raise HTTPLookupError('HTTP lookup failed for {0}.'.format(
                    url))

        except HTTPLookupError as e:  # pragma: no cover

            raise e

        except Exception:  # pragma: no cover

            raise HTTPLookupError('HTTP lookup failed for {0}.'.format(url))



#!/usr/bin/env python
#coding:utf-8
"""
  Author:   --<v1ll4n>
  Purpose: process_task for a task(ProcessMode)
  Created: 2016/12/12
"""

import unittest
import time
import multiprocessing
import threading
from multiprocessing import Pipe
from pprint import pprint

from . import exceptions

def sleep_(num):
    #pprint("~~~")
    time.sleep(num)

#----------------------------------------------------------------------
def testfun(num):
    """"""
    #print('UserFunc called!')
    for i in range(6):
        threading.Thread(target=sleep_, args=(num,)).start()
        #print('SubProcess Called!')
    
    time.sleep(0.4)
    
    #pprint(threading.enumerate())

########################################################################
class ProcessTask(multiprocessing.Process):
    """"""

    #----------------------------------------------------------------------
    def __init__(self, id, target, args=tuple(), kwargs={}, status_monitor_pipe=None,
                 threads_update_interval=0.0):
        """Constructor"""
        
        multiprocessing.Process.__init__(self, name=id)
        
        self._target = target
        self.args = args
        self.kwargs = kwargs
        
        self._id = id
        
        self._sub_threads_list = []
    
        self._threads_update_interval = threads_update_interval
                
        self._status_monitor_pipe = status_monitor_pipe
        #self._init_timer()
    
    #----------------------------------------------------------------------
    def _init_timer(self):
        """"""
        self._threads_monitor = threading.Thread(name='update_subthreads_list',
                                                 target=self._deamon_check_threads)
        self._threads_monitor.daemon = True
        self._threads_monitor.start()
        
    
    #----------------------------------------------------------------------
    @property    
    def task_id(self):
        """"""
        return self._id
    
    #----------------------------------------------------------------------
    def run(self):
        """"""
        self._init_timer()
        try:
            self._target(*self.args, **self.kwargs)
        except exceptions.TaskRuntimeError:
            raise NotImplemented
        
    #----------------------------------------------------------------------
    def _enum_threads(self):
        """"""
        threads_list = threading.enumerate()
        return threads_list
    
    #----------------------------------------------------------------------
    def _deamon_check_threads(self):
        """"""
        assert isinstance(self._threads_update_interval, (int, float))
        while True:
            #pprint('test')
            
            self._sub_threads_list = None
            self._sub_threads_list = self._enum_threads()
            #print(len(self._sub_threads_list))
            #print len(self._sub_threads_list)
            threads_check_result = {}
            threads_check_result['from'] = self._id
            for i in self._sub_threads_list:
                threads_check_result[i.name] = i.is_alive()
                #pprint(threads_check_result)
                
            self._status_monitor_pipe.send(threads_check_result)
            time.sleep(self._threads_update_interval)
            
    ##----------------------------------------------------------------------
    #@property    
    #def subthreads_count(self):
        #return len(self._sub_threads_list)
        
            
        

########################################################################
class ProcessTaskTest(unittest.case.TestCase):
    """"""

    #----------------------------------------------------------------------
    def print_bar(self):
        """"""
        print(('-'*64))
    
    #----------------------------------------------------------------------
    def print_end_bar(self):
        """"""
        print(('-'*30 + 'END' + '-'*31))
        
        

    #----------------------------------------------------------------------
    def runTest(self):
        """"""
        pipp, pipc = Pipe()
        self.print_bar()
        print('Test Task Interface')
        ret_process = ProcessTask(id='test-1', target=testfun, args=(5,), 
                                  status_monitor_pipe=pipc)
        ret_process.start()
        print('Test get threads status')
        time.sleep(1)
        #print(ret_process.subthreads_count)
        
        
        threads_status = pipp.recv()
        self.assertIsInstance(threads_status, dict)
        self.print_end_bar()
        
        
    
    
        
if __name__ == '__main__':
    unittest.main()


# Copyright (c) 2013, 2014, 2015, 2016 Philip Hane
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from . import (Net, NetError, InvalidEntityContactObject, InvalidNetworkObject,
               InvalidEntityObject, HTTPLookupError)
from .utils import ipv4_lstrip_zeros, calculate_cidr, unique_everseen
from .net import ip_address
import logging
import json

log = logging.getLogger(__name__)

BOOTSTRAP_URL = 'http://rdap.arin.net/bootstrap'

RIR_RDAP = {
    'arin': {
        'ip_url': 'http://rdap.arin.net/registry/ip/{0}',
        'entity_url': 'http://rdap.arin.net/registry/entity/{0}'
    },
    'ripencc': {
        'ip_url': 'http://rdap.db.ripe.net/ip/{0}',
        'entity_url': 'http://rdap.db.ripe.net/entity/{0}'
    },
    'apnic': {
        'ip_url': 'http://rdap.apnic.net/ip/{0}',
        'entity_url': 'http://rdap.apnic.net/entity/{0}'
    },
    'lacnic': {
        'ip_url': 'http://rdap.lacnic.net/rdap/ip/{0}',
        'entity_url': 'http://rdap.lacnic.net/rdap/entity/{0}'
    },
    'afrinic': {
        'ip_url': 'http://rdap.afrinic.net/rdap/ip/{0}',
        'entity_url': 'http://rdap.afrinic.net/rdap/entity/{0}'
    }
}


class _RDAPContact:
    """
    The class for parsing RDAP entity contact information objects:
    https://tools.ietf.org/html/rfc7483#section-5.1
    https://tools.ietf.org/html/rfc7095

    Args:
        vcard: The vcard list from an RDAP IP address query.

    Raises:
        InvalidEntityContactObject: vcard is not an RDAP entity contact
            information object.
    """

    def __init__(self, vcard):

        if not isinstance(vcard, list):

            raise InvalidEntityContactObject('JSON result must be a list.')

        self.vcard = vcard
        self.vars = {
            'name': None,
            'kind': None,
            'address': None,
            'phone': None,
            'email': None,
            'role': None,
            'title': None
        }

    def _parse_name(self, val):
        """
        The function for parsing the vcard name.

        Args:
            val: The value to parse.
        """

        self.vars['name'] = val[3].strip()

    def _parse_kind(self, val):
        """
        The function for parsing the vcard kind.

        Args:
            val: The value to parse.
        """

        self.vars['kind'] = val[3].strip()

    def _parse_address(self, val):
        """
        The function for parsing the vcard address.

        Args:
            val: The value to parse.
        """

        ret = {
            'type': None,
            'value': None
        }

        try:

            ret['type'] = val[1]['type']

        except (KeyError, ValueError, TypeError):

                pass

        try:

            ret['value'] = val[1]['label']

        except (KeyError, ValueError, TypeError):

            ret['value'] = '\n'.join(val[3]).strip()

        try:

            self.vars['address'].append(ret)

        except AttributeError:

            self.vars['address'] = []
            self.vars['address'].append(ret)

    def _parse_phone(self, val):
        """
        The function for parsing the vcard phone numbers.

        Args:
            val: The value to parse.
        """

        ret = {
            'type': None,
            'value': None
        }

        try:

            ret['type'] = val[1]['type']

        except (IndexError, KeyError, ValueError, TypeError):

                pass

        ret['value'] = val[3].strip()

        try:

            self.vars['phone'].append(ret)

        except AttributeError:

            self.vars['phone'] = []
            self.vars['phone'].append(ret)

    def _parse_email(self, val):
        """
        The function for parsing the vcard email addresses.

        Args:
            val: The value to parse.
        """

        ret = {
            'type': None,
            'value': None
        }

        try:

            ret['type'] = val[1]['type']

        except (KeyError, ValueError, TypeError):

                pass

        ret['value'] = val[3].strip()

        try:

            self.vars['email'].append(ret)

        except AttributeError:

            self.vars['email'] = []
            self.vars['email'].append(ret)

    def _parse_role(self, val):
        """
        The function for parsing the vcard role.

        Args:
            val: The value to parse.
        """

        self.vars['role'] = val[3].strip()

    def _parse_title(self, val):
        """
        The function for parsing the vcard title.

        Args:
            val: The value to parse.
        """

        self.vars['title'] = val[3].strip()

    def parse(self):
        """
        The function for parsing the vcard to the vars dictionary.
        """

        keys = {
            'fn': self._parse_name,
            'kind': self._parse_kind,
            'adr': self._parse_address,
            'tel': self._parse_phone,
            'email': self._parse_email,
            'role': self._parse_role,
            'title': self._parse_title
        }

        for val in self.vcard:

            try:

                parser = keys.get(val[0])
                parser(val)

            except (KeyError, ValueError, TypeError):

                pass


class _RDAPCommon:
    """
    The common class for parsing RDAP objects:
    https://tools.ietf.org/html/rfc7483#section-5

    Args:
        json_result: The JSON response from an RDAP query.

    Raises:
        ValueError: vcard is not a known RDAP object.
    """

    def __init__(self, json_result):

        if not isinstance(json_result, dict):

            raise ValueError

        self.json = json_result
        self.vars = {
            'handle': None,
            'status': None,
            'remarks': None,
            'notices': None,
            'links': None,
            'events': None,
            'raw': None
        }

    def summarize_links(self, links_json):
        """
        The function for summarizing RDAP links in to a unique list.
        https://tools.ietf.org/html/rfc7483#section-4.2

        Args:
            links_json: A json dictionary of links from RDAP results.

        Returns:
            List: A unique list of found RDAP link dictionaries.
        """

        ret = []

        for link_dict in links_json:

            ret.append(link_dict['href'])

        ret = list(unique_everseen(ret))

        return ret

    def summarize_notices(self, notices_json):
        """
        The function for summarizing RDAP notices in to a unique list.
        https://tools.ietf.org/html/rfc7483#section-4.3

        Args:
            notices_json: A json dictionary of notices from RDAP results.

        Returns:
            List: A unique list of found RDAP notices dictionaries.
        """

        ret = []

        for notices_dict in notices_json:

            tmp = {
                'title': None,
                'description': None,
                'links': None
            }

            try:

                tmp['title'] = notices_dict['title']

            except (KeyError, ValueError, TypeError):

                pass

            try:

                tmp['description'] = '\n'.join(notices_dict['description'])

            except (KeyError, ValueError, TypeError):

                pass

            try:

                tmp['links'] = self.summarize_links(notices_dict['links'])

            except (KeyError, ValueError, TypeError):

                pass

            if all(tmp.values()):

                ret.append(tmp)

        return ret

    def summarize_events(self, events_json):
        """
        The function for summarizing RDAP events in to a unique list.
        https://tools.ietf.org/html/rfc7483#section-4.5

        Args:
            events_json: A json dictionary of events from RDAP results.

        Returns:
            List: A unique list of found RDAP events dictionaries.
        """

        ret = []

        for event in events_json:

            event_dict = {
                'action': event['eventAction'],
                'timestamp': event['eventDate'],
                'actor': None
            }

            try:

                event_dict['actor'] = event['eventActor']

            except (KeyError, ValueError, TypeError):

                pass

            ret.append(event_dict)

        return ret

    def _parse(self):
        """
        The function for parsing the JSON response to the vars dictionary.
        """

        try:

            self.vars['status'] = self.json['status']

        except (KeyError, ValueError, TypeError):

            pass

        for v in ['remarks', 'notices']:

            try:

                self.vars[v] = self.summarize_notices(self.json[v])

            except (KeyError, ValueError, TypeError):

                pass

        try:

            self.vars['links'] = self.summarize_links(self.json['links'])

        except (KeyError, ValueError, TypeError):

            pass

        try:

            self.vars['events'] = self.summarize_events(self.json['events'])

        except (KeyError, ValueError, TypeError):

            pass


class _RDAPNetwork(_RDAPCommon):
    """
    The class for parsing RDAP network objects:
    https://tools.ietf.org/html/rfc7483#section-5.4

    Args:
        json_result: The JSON response from an RDAP IP address query.

    Raises:
        InvalidNetworkObject: json_result is not an RDAP network object.
    """

    def __init__(self, json_result):

        try:

            _RDAPCommon.__init__(self, json_result)

        except ValueError:

            raise InvalidNetworkObject('JSON result must be a dict.')

        self.vars.update({
            'start_address': None,
            'end_address': None,
            'cidr': None,
            'ip_version': None,
            'type': None,
            'name': None,
            'country': None,
            'parent_handle': None
        })

    def parse(self):
        """
        The function for parsing the JSON response to the vars dictionary.
        """

        try:

            self.vars['handle'] = self.json['handle'].strip()

        except (KeyError, ValueError):

            log.debug('Handle missing, json_output: {0}'.format(json.dumps(
                self.json)))
            raise InvalidNetworkObject('Handle is missing for RDAP network '
                                       'object')

        try:

            self.vars['ip_version'] = self.json['ipVersion'].strip()

            # RDAP IPv4 addresses are padded to 3 digits per octet, remove
            # the leading 0's.
            if self.vars['ip_version'] == 'v4':

                self.vars['start_address'] = ip_address(
                    ipv4_lstrip_zeros(self.json['startAddress'])
                ).__str__()

                self.vars['end_address'] = ip_address(
                    ipv4_lstrip_zeros(self.json['endAddress'])
                ).__str__()

            # No bugs found for IPv6 yet, proceed as normal.
            else:

                self.vars['start_address'] = self.json['startAddress'].strip()
                self.vars['end_address'] = self.json['endAddress'].strip()

        except (KeyError, ValueError, TypeError):

            log.debug('IP address data incomplete. Data parsed prior to '
                      'exception: {0}'.format(json.dumps(self.vars)))
            raise InvalidNetworkObject('IP address data is missing for RDAP '
                                       'network object.')

        try:

            self.vars['cidr'] = ', '.join(calculate_cidr(
                self.vars['start_address'], self.vars['end_address']
            ))

        except (KeyError, ValueError, TypeError, AttributeError) as \
                e:  # pragma: no cover

            log.debug('CIDR calculation failed: {0}'.format(e))
            pass

        for v in ['name', 'type', 'country']:

            try:

                self.vars[v] = self.json[v].strip()

            except (KeyError, ValueError):

                pass

        try:

            self.vars['parent_handle'] = self.json['parentHandle'].strip()

        except (KeyError, ValueError):

            pass

        self._parse()


class _RDAPEntity(_RDAPCommon):
    """
    The class for parsing RDAP entity objects:
    https://tools.ietf.org/html/rfc7483#section-5.1

    Args:
        json_result: The JSON response from an RDAP query.

    Raises:
        InvalidEntityObject: json_result is not an RDAP entity object.
    """

    def __init__(self, json_result):

        try:

            _RDAPCommon.__init__(self, json_result)

        except ValueError:

            raise InvalidEntityObject('JSON result must be a dict.')

        self.vars.update({
            'roles': None,
            'contact': None,
            'events_actor': None,
            'entities': []
        })

    def parse(self):
        """
        The function for parsing the JSON response to the vars dictionary.
        """

        try:

            self.vars['handle'] = self.json['handle'].strip()

        except (KeyError, ValueError, TypeError):

            raise InvalidEntityObject('Handle is missing for RDAP entity')

        for v in ['roles', 'country']:

            try:

                self.vars[v] = self.json[v]

            except (KeyError, ValueError):

                pass

        try:

            vcard = self.json['vcardArray'][1]
            c = _RDAPContact(vcard)
            c.parse()

            self.vars['contact'] = c.vars

        except (KeyError, ValueError, TypeError):

            pass

        try:

            self.vars['events_actor'] = self.summarize_events(
                self.json['asEventActor'])

        except (KeyError, ValueError, TypeError):

            pass

        self.vars['entities'] = []
        try:

            for ent in self.json['entities']:

                if ent['handle'] not in self.vars['entities']:

                    self.vars['entities'].append(ent['handle'])

        except (KeyError, ValueError, TypeError):

            pass

        if not self.vars['entities']:

            self.vars['entities'] = None

        self._parse()


class RDAP:
    """
    The class for parsing IP address whois information via RDAP:
    https://tools.ietf.org/html/rfc7483
    https://www.arin.net/resources/rdap.html

    Args:
        net: A ipwhois.net.Net object.

    Raises:
        NetError: The parameter provided is not an instance of
            ipwhois.net.Net
        IPDefinedError: The address provided is defined (does not need to be
            resolved).
    """

    def __init__(self, net):

        if isinstance(net, Net):

            self._net = net

        else:

            raise NetError('The provided net parameter is not an instance of '
                           'ipwhois.net.Net')

    def lookup(self, inc_raw=False, retry_count=3, asn_data=None, depth=0,
               excluded_entities=None, response=None, bootstrap=False,
               rate_limit_timeout=120):
        """
        The function for retrieving and parsing information for an IP
        address via RDAP (HTTP).

        Args:
            inc_raw: Boolean for whether to include the raw results in the
                returned dictionary.
            retry_count: The number of times to retry in case socket errors,
                timeouts, connection resets, etc. are encountered.
            asn_data: Result dictionary from ipwhois.net.Net.lookup_asn().
                Optional if the bootstrap parameter is True.
            depth: How many levels deep to run queries when additional
                referenced objects are found.
            excluded_entities: A list of entity handles to not perform lookups.
            response: Optional response object, this bypasses the RDAP lookup.
            bootstrap: If True, performs lookups via ARIN bootstrap rather
                than lookups based on ASN data.
            rate_limit_timeout: The number of seconds to wait before retrying
                when a rate limit notice is returned via rdap+json.

        Returns:
            Dictionary:

            :query: The IP address (String)
            :network: Dictionary of values returned by _RDAPNetwork. The raw
                result is included for each entity if the inc_raw parameter is
                True.
            :entities: List of entity keys referenced by the top level IP
                address query.
            :objects: Dictionary of objects with the handles as keys, and the
                dictionary returned by _RDAPEntity, etc as the values. The raw
                result is included for each object if the inc_raw parameter is
                True.
        """

        if not excluded_entities:

            excluded_entities = []

        # Create the return dictionary.
        results = {
            'query': self._net.address_str,
            'network': None,
            'entities': None,
            'objects': None,
            'raw': None
        }

        if bootstrap:

            ip_url = '{0}/ip/{1}'.format(BOOTSTRAP_URL, self._net.address_str)

        else:

            ip_url = str(RIR_RDAP[asn_data['asn_registry']]['ip_url']).format(
                self._net.address_str)

        # Only fetch the response if we haven't already.
        if response is None:

            log.debug('Response not given, perform RDAP lookup for {0}'.format(
                ip_url))

            # Retrieve the whois data.
            response = self._net.get_http_json(
                url=ip_url, retry_count=retry_count,
                rate_limit_timeout=rate_limit_timeout
            )

        if inc_raw:

            results['raw'] = response

        log.debug('Parsing RDAP network object')
        result_net = _RDAPNetwork(response)
        result_net.parse()
        results['network'] = result_net.vars
        results['entities'] = []
        results['objects'] = {}

        # Iterate through and parse the root level entities.
        log.debug('Parsing RDAP root level entities')
        try:

            for ent in response['entities']:

                if ent['handle'] not in [results['entities'],
                                         excluded_entities]:

                    result_ent = _RDAPEntity(ent)
                    result_ent.parse()

                    results['objects'][ent['handle']] = result_ent.vars

                    results['entities'].append(ent['handle'])

        except KeyError:

            pass

        # Iterate through to the defined depth, retrieving and parsing all
        # unique entities.
        temp_objects = results['objects']

        if depth > 0 and len(temp_objects) > 0:

            log.debug('Parsing RDAP sub-entities to depth: {0}'.format(str(
                depth)))

        while depth > 0 and len(temp_objects) > 0:

            new_objects = {}
            for obj in list(temp_objects.values()):

                try:

                    for ent in obj['entities']:

                        if ent not in (list(results['objects'].keys()) +
                                       list(new_objects.keys()) +
                                       excluded_entities):

                            if bootstrap:
                                entity_url = '{0}/entity/{1}'.format(
                                    BOOTSTRAP_URL, ent)
                            else:
                                tmp_reg = asn_data['asn_registry']
                                entity_url = RIR_RDAP[tmp_reg]['entity_url']
                                entity_url = str(entity_url).format(ent)

                            try:

                                # RDAP entity query
                                response = self._net.get_http_json(
                                    url=entity_url, retry_count=retry_count,
                                    rate_limit_timeout=rate_limit_timeout
                                )

                                # Parse the entity
                                result_ent = _RDAPEntity(response)
                                result_ent.parse()
                                new_objects[ent] = result_ent.vars

                                if inc_raw:

                                    new_objects[ent]['raw'] = response

                            except (HTTPLookupError, InvalidEntityObject):

                                pass

                except TypeError:

                    pass

            # Update the result objects, and set the new temp object list to
            # iterate for the next depth of entities.
            results['objects'].update(new_objects)
            temp_objects = new_objects
            depth -= 1

        return results



#!/usr/bin/env python
#coding:utf-8
"""
  Author:   --<v1ll4n>
  Purpose: Python Ping
  Created: 2016/12/26
"""

import unittest
from .isalive import is_alive

#----------------------------------------------------------------------
def pyping(target, timeout=2, count=4):
    """"""
    return is_alive(target)
    

if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python

"""
    A pure python ping implementation using raw socket.


    Note that ICMP messages can only be sent from processes running as root.


    Derived from ping.c distributed in Linux's netkit. That code is
    copyright (c) 1989 by The Regents of the University of California.
    That code is in turn derived from code written by Mike Muuss of the
    US Army Ballistic Research Laboratory in December, 1983 and
    placed in the public domain. They have my thanks.

    Bugs are naturally mine. I'd be glad to hear about them. There are
    certainly word - size dependenceies here.

    Copyright (c) Matthew Dixon Cowles, <http://www.visi.com/~mdc/>.
    Distributable under the terms of the GNU General Public License
    version 2. Provided with no warranties of any sort.

    Original Version from Matthew Dixon Cowles:
      -> ftp://ftp.visi.com/users/mdc/ping.py

    Rewrite by Jens Diemer:
      -> http://www.python-forum.de/post-69122.html#69122


    Revision history
    ~~~~~~~~~~~~~~~~

    March 11, 2010
    changes by Samuel Stauffer:
    - replaced time.clock with default_timer which is set to
      time.clock on windows and time.time on other systems.

    May 30, 2007
    little rewrite by Jens Diemer:
     -  change socket asterisk import to a normal import
     -  replace time.time() with time.clock()
     -  delete "return None" (or change to "return" only)
     -  in checksum() rename "str" to "source_string"

    November 22, 1997
    Initial hack. Doesn't do much, but rather than try to guess
    what features I (or others) will want in the future, I've only
    put in what I need now.

    December 16, 1997
    For some reason, the checksum bytes are in the wrong order when
    this is run under Solaris 2.X for SPARC but it works right under
    Linux x86. Since I don't know just what's wrong, I'll swap the
    bytes always and then do an htons().

    December 4, 2000
    Changed the struct.pack() calls to pack the checksum and ID as
    unsigned. My thanks to Jerome Poincheval for the fix.

    Januari 27, 2015
    Changed receive response to not accept ICMP request messages.
    It was possible to receive the very request that was sent.

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate: $
    $Rev: $
    $Author: $
"""


import os, sys, socket, struct, select, time

if sys.platform == "win32":
    # On Windows, the best timer is time.clock()
    default_timer = time.clock
else:
    # On most other platforms the best timer is time.time()
    default_timer = time.time

# From /usr/include/linux/icmp.h; your milage may vary.
ICMP_ECHO_REQUEST = 8 # Seems to be the same on Solaris.


def checksum(source_string):
    """
    I'm not too confident that this is right but testing seems
    to suggest that it gives the same answers as in_cksum in ping.c
    """
    sum = 0
    countTo = (len(source_string)/2)*2
    count = 0
    while count<countTo:
        thisVal = ord(source_string[count + 1])*256 + ord(source_string[count])
        sum = sum + thisVal
        sum = sum & 0xffffffff # Necessary?
        count = count + 2

    if countTo<len(source_string):
        sum = sum + ord(source_string[len(source_string) - 1])
        sum = sum & 0xffffffff # Necessary?

    sum = (sum >> 16)  +  (sum & 0xffff)
    sum = sum + (sum >> 16)
    answer = ~sum
    answer = answer & 0xffff

    # Swap bytes. Bugger me if I know why.
    answer = answer >> 8 | (answer << 8 & 0xff00)

    return answer


def receive_one_ping(my_socket, ID, timeout):
    """
    receive the ping from the socket.
    """
    timeLeft = timeout
    while True:
        startedSelect = default_timer()
        whatReady = select.select([my_socket], [], [], timeLeft)
        howLongInSelect = (default_timer() - startedSelect)
        if whatReady[0] == []: # Timeout
            return

        timeReceived = default_timer()
        recPacket, addr = my_socket.recvfrom(1024)
        icmpHeader = recPacket[20:28]
        type, code, checksum, packetID, sequence = struct.unpack(
            "bbHHh", icmpHeader
        )
        # Filters out the echo request itself. 
        # This can be tested by pinging 127.0.0.1 
        # You'll see your own request
        if type != 8 and packetID == ID:
            bytesInDouble = struct.calcsize("d")
            timeSent = struct.unpack("d", recPacket[28:28 + bytesInDouble])[0]
            return timeReceived - timeSent

        timeLeft = timeLeft - howLongInSelect
        if timeLeft <= 0:
            return


def send_one_ping(my_socket, dest_addr, ID):
    """
    Send one ping to the given >dest_addr<.
    """
    dest_addr  =  socket.gethostbyname(dest_addr)

    # Header is type (8), code (8), checksum (16), id (16), sequence (16)
    my_checksum = 0

    # Make a dummy heder with a 0 checksum.
    header = struct.pack("bbHHh", ICMP_ECHO_REQUEST, 0, my_checksum, ID, 1)
    bytesInDouble = struct.calcsize("d")
    data = (192 - bytesInDouble) * "Q"
    data = struct.pack("d", default_timer()) + data

    # Calculate the checksum on the data and the dummy header.
    my_checksum = checksum(header + data)

    # Now that we have the right checksum, we put that in. It's just easier
    # to make up a new header than to stuff it into the dummy.
    header = struct.pack(
        "bbHHh", ICMP_ECHO_REQUEST, 0, socket.htons(my_checksum), ID, 1
    )
    packet = header + data
    my_socket.sendto(packet, (dest_addr, 1)) # Don't know about the 1


def do_one(dest_addr, timeout):
    """
    Returns either the delay (in seconds) or none on timeout.
    """
    icmp = socket.getprotobyname("icmp")
    try:
        my_socket = socket.socket(socket.AF_INET, socket.SOCK_RAW, icmp)
    except socket.error as xxx_todo_changeme:
        (errno, msg) = xxx_todo_changeme.args
        if errno == 1:
            # Operation not permitted
            msg = msg + (
                " - Note that ICMP messages can only be sent from processes"
                " running as root."
            )
            raise socket.error(msg)
        raise # raise the original error

    my_ID = os.getpid() & 0xFFFF

    send_one_ping(my_socket, dest_addr, my_ID)
    delay = receive_one_ping(my_socket, my_ID, timeout)

    my_socket.close()
    return delay


def verbose_ping(dest_addr, timeout = 2, count = 4):
    """
    Send >count< ping to >dest_addr< with the given >timeout< and display
    the result.
    """
    delay_ave = 0
    for i in range(count):
        #print "ping %s..." % dest_addr,
        try:
            delay  =  do_one(dest_addr, timeout)
        except socket.gaierror as e:
            #print "failed. (socket error: '%s')" % e[1]
            break

        if delay  ==  None:
            #print "failed. (timeout within %ssec.)" % timeout
            pass
        else:
            delay  =  delay * 1000
            delay_ave = (delay_ave + delay) / 2
            #print "get ping in %0.4fms" % delay
    return delay_ave


if __name__ == '__main__':
    verbose_ping("heise.de")
    verbose_ping("google.com")
    verbose_ping("a-test-url-taht-is-not-available.com")
    verbose_ping("192.168.1.1")



#!/usr/bin/env python
#coding:utf-8
"""
  Author:   --<v1ll4n>
  Purpose: Manage Task
  Created: 2016/12/13
"""

from time import sleep
import unittest
from multiprocessing import Pipe
from threading import Thread
from pprint import pprint
from inspect import getmembers

from . import exceptions
from .process_task import ProcessTask
from .utils_class import Singleton
from .process_task import testfun



UPDATE_TASK_STATUS_DAEMON = 'update_task_status_daemon'


########################################################################
class TaskBulter(Singleton):
    """"""

    _tasks_table = {}
    _tasks_status = {}
    _daemon_threads = {}
    
    #----------------------------------------------------------------------
    def __init__(self, threads_update_interval=0):
        """Constructor"""
        
        self._threads_update_interval = threads_update_interval
        self._closed = False
        self._initial_deamon_threads()
    
    #----------------------------------------------------------------------
    def _daemon_start(self, name, func):
        """"""
        if name in self._daemon_threads:
            pass
        else:
            ret = Thread(name=name, target=func)
            ret.daemon = True
            ret.start()
            self._daemon_threads[name] = ret

    #----------------------------------------------------------------------
    def _initial_deamon_threads(self):
        """"""
        #raise NotImplemented
    
        self._daemon_start(name=UPDATE_TASK_STATUS_DAEMON,
                     func=self._update_tasks_status)
        
    
    #----------------------------------------------------------------------
    def _update_tasks_status(self):
        """"""
        #pprint('daemon threads started')
        while self._closed == False:
            for i in list(self._tasks_table.items()):
                pipe = i[1]['status_monitor_pipe']
                if self._tasks_table[i[0]]['process_instance'].exitcode == None:
                    if pipe.poll():
                        self._tasks_status[i[0]] = pipe.recv()
                        #pprint(self._tasks_status)
                    else:
                        pass
                    sleep(self._threads_update_interval/2)
                else:
                    self._tasks_status[i[0]] = {}
    
    
    #----------------------------------------------------------------------
    def start_task(cls, id, target, args=tuple(), kwargs={},):
        """Start A task(Process)
        
        Params:
            id: the ID of task (identify the process_task)
              :type: str
            target: the task function
              :type: function
            args: the vargs of target
              :type: tuple
            kwargs: the keywords of target
              :type: dict"""
        
        if not callable(target): exceptions.TaskCannotBeCalled
        
        if id in cls._tasks_table:
            raise exceptions.ExistedTaskId
        
        control_pipe, child_pipe = Pipe(duplex=False)
        
        cls._tasks_table[id] = {}
        cls._tasks_table[id]['status_monitor_pipe'] = control_pipe
        task_process = ProcessTask(id, target, args=args, kwargs=kwargs, 
                                   status_monitor_pipe=child_pipe, 
                                   threads_update_interval=cls._threads_update_interval)
        
        cls._tasks_table[id]['process_instance'] = task_process
        task_process.daemon = True
        task_process.start()
    
    
    #----------------------------------------------------------------------
    def get_tasks_status(self):
        """"""
        return self._tasks_status.copy()
    
    #----------------------------------------------------------------------
    def get_tasks(self):
        """"""
        return self._tasks_table
    
    #----------------------------------------------------------------------
    def get_task_by_id(self, id):
        """"""
        if id in self._tasks_table:
            return self._tasks_table[id]
        else:
            return None
    
    #----------------------------------------------------------------------
    def destory_task(self, id_or_taskinstance):
        """"""
        if isinstance(id_or_taskinstance, ProcessTask):
            id_or_taskinstance.terminate()
        elif isinstance(id_or_taskinstance, str):
            _ = self.get_task_by_id(id_or_taskinstance)['process_instance']
            assert isinstance(_, ProcessTask)
            _.terminate()
        
    #----------------------------------------------------------------------
    def close(self):
        """"""
        self._closed = True
        
########################################################################
class TaskBulterTest(unittest.case.TestCase):
    """"""

    #----------------------------------------------------------------------
    def test_add_task_and_kill_task(self):
        """Constructor"""
        TaskBulter().start_task(id='test-1', target=testfun, args=(6,))
        #sleep(1)
        for i in range(3):
            pprint(TaskBulter().get_task_status())
            
            self.assertIsInstance(TaskBulter().get_task_by_id('test-1')['process_instance'], ProcessTask)
            processi = TaskBulter().get_task_by_id('test-1')['process_instance']
            sleep(1)
        
        TaskBulter().start_task(id='test-2', target=testfun, args=(6,))
        processi = TaskBulter().get_task_by_id('test-2')['process_instance']
        #processi.terminate()
        TaskBulter().destory_task('test-2')
        
        for i in range(3):
            pprint(TaskBulter().get_task_status())
            self.assertIsInstance(TaskBulter().get_task_by_id('test-2')['process_instance'], ProcessTask)
            sleep(1)        
    
    


if __name__ == '__main__':
    unittest.main()


# Copyright (c) 2013, 2014, 2015, 2016 Philip Hane
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# TODO: Add '_links' for RFC/other references

HR_ASN = {
    'asn': {
        '_short': 'ASN',
        '_name': 'Autonomous System Number',
        '_description': 'Globally unique identifier used for routing '
                        'information exchange with Autonomous Systems.'
    },
    'asn_cidr': {
        '_short': 'ASN CIDR Block',
        '_name': 'ASN Classless Inter-Domain Routing Block',
        '_description': 'Network routing block assigned to an ASN.'
    },
    'asn_country_code': {
        '_short': 'ASN Country Code',
        '_name': 'ASN Assigned Country Code',
        '_description': 'ASN assigned country code in ISO 3166-1 format.'
    },
    'asn_date': {
        '_short': 'ASN Date',
        '_name': 'ASN Allocation Date',
        '_description': 'ASN allocation date in ISO 8601 format.'
    },
    'asn_registry': {
        '_short': 'ASN Registry',
        '_name': 'ASN Assigned Registry',
        '_description': 'ASN assigned regional internet registry.'
    }
}

HR_RDAP_COMMON = {
    'entities': {
        '_short': 'Entities',
        '_name': 'RIR Object Entities',
        '_description': 'List of object names referenced by an RIR object.'
    },
    'events': {
        '_short': 'Events',
        '_name': 'Events',
        '_description': 'Events for an RIR object.',
        'action': {
            '_short': 'Action',
            '_name': 'Event Action (Reason)',
            '_description': 'The reason for an event.'
        },
        'timestamp': {
            '_short': 'Timestamp',
            '_name': 'Event Timestamp',
            '_description': 'The date an event occured in ISO 8601 '
                            'format.'
        },
        'actor': {
            '_short': 'Actor',
            '_name': 'Event Actor',
            '_description': 'The identifier for an event initiator.'
        }
    },
    'handle': {
        '_short': 'Handle',
        '_name': 'RIR Handle',
        '_description': 'Unique identifier for a registered object.'
    },
    'links': {
        '_short': 'Links',
        '_name': 'Links',
        '_description': 'HTTP/HTTPS links provided for an RIR object.'
    },
    'notices': {
        '_short': 'Notices',
        '_name': 'Notices',
        '_description': 'Notices for an RIR object.',
        'description': {
            '_short': 'Description',
            '_name': 'Notice Description',
            '_description': 'The description/body of a notice.'
        },
        'title': {
            '_short': 'Title',
            '_name': 'Notice Title',
            '_description': 'The title/header for a notice.'
        },
        'links': {
            '_short': 'Links',
            '_name': 'Notice Links',
            '_description': 'HTTP/HTTPS links provided for a notice.'
        }
    },
    'remarks': {
        '_short': 'Remarks',
        '_name': 'Remarks',
        '_description': 'Remarks for an RIR object.',
        'description': {
            '_short': 'Description',
            '_name': 'Remark Description',
            '_description': 'The description/body of a remark.'
        },
        'title': {
            '_short': 'Title',
            '_name': 'Remark Title',
            '_description': 'The title/header for a remark.'
        },
        'links': {
            '_short': 'Links',
            '_name': 'Remark Links',
            '_description': 'HTTP/HTTPS links provided for a remark.'
        }
    },
    'status': {
        '_short': 'Status',
        '_name': 'Object Status',
        '_description': 'List indicating the state of a registered object.'
    }
}

HR_RDAP = {
    'network': {
        '_short': 'Network',
        '_name': 'RIR Network',
        '_description': 'The assigned network for an IP address.',
        'cidr': {
            '_short': 'CIDR Block',
            '_name': 'Classless Inter-Domain Routing Block',
            '_description': 'Network routing block an IP address belongs to.'
        },
        'country': {
            '_short': 'Country Code',
            '_name': 'Country Code',
            '_description': 'Country code registered with the RIR in '
                            'ISO 3166-1 format.'
        },
        'end_address': {
            '_short': 'End Address',
            '_name': 'Ending IP Address',
            '_description': 'The last IP address in a network block.'
        },
        'events': HR_RDAP_COMMON['events'],
        'handle': HR_RDAP_COMMON['handle'],
        'ip_version': {
            '_short': 'IP Version',
            '_name': 'IP Protocol Version',
            '_description': 'The IP protocol version (v4 or v6) of an IP '
                            'address.'
        },
        'links': HR_RDAP_COMMON['links'],
        'name': {
            '_short': 'Name',
            '_name': 'RIR Network Name',
            '_description': 'The identifier assigned to the network '
                            'registration for an IP address.'
        },
        'notices': HR_RDAP_COMMON['notices'],
        'parent_handle': {
            '_short': 'Parent Handle',
            '_name': 'RIR Parent Handle',
            '_description': 'Unique identifier for the parent network of '
                            'a registered network.'
        },
        'remarks': HR_RDAP_COMMON['remarks'],
        'start_address': {
            '_short': 'Start Address',
            '_name': 'Starting IP Address',
            '_description': 'The first IP address in a network block.'
        },
        'status': HR_RDAP_COMMON['status'],
        'type': {
            '_short': 'Type',
            '_name': 'RIR Network Type',
            '_description': 'The RIR classification of a registered network.'
        }
    },
    'entities': HR_RDAP_COMMON['entities'],
    'objects': {
        '_short': 'Objects',
        '_name': 'RIR Objects',
        '_description': 'The objects (entities) referenced by an RIR network.',
        'contact': {
            '_short': 'Contact',
            '_name': 'Contact Information',
            '_description': 'Contact information registered with an RIR '
                            'object.',
            'address': {
                '_short': 'Address',
                '_name': 'Postal Address',
                '_description': 'The contact postal address.'
            },
            'email': {
                '_short': 'Email',
                '_name': 'Email Address',
                '_description': 'The contact email address.'
            },
            'kind': {
                '_short': 'Kind',
                '_name': 'Kind',
                '_description': 'The contact information kind (individual, '
                                'group, org, etc).'
            },
            'name': {
                '_short': 'Name',
                '_name': 'Name',
                '_description': 'The contact name.'
            },
            'phone': {
                '_short': 'Phone',
                '_name': 'Phone Number',
                '_description': 'The contact phone number.'
            },
            'role': {
                '_short': 'Role',
                '_name': 'Role',
                '_description': 'The contact\'s role.'
            },
            'title': {
                '_short': 'Title',
                '_name': 'Title',
                '_description': 'The contact\'s position or job title.'
            }
        },
        'entities': HR_RDAP_COMMON['entities'],
        'events': HR_RDAP_COMMON['events'],
        'events_actor': {
            '_short': 'Events Misc',
            '_name': 'Events w/o Actor',
            '_description': 'An event for an RIR object with no event actor.',
            'action': {
                '_short': 'Action',
                '_name': 'Event Action (Reason)',
                '_description': 'The reason for an event.'
            },
            'timestamp': {
                '_short': 'Timestamp',
                '_name': 'Event Timestamp',
                '_description': 'The date an event occured in ISO 8601 '
                                'format.'
            }
        },
        'handle': HR_RDAP_COMMON['handle'],
        'links': HR_RDAP_COMMON['links'],
        'notices': HR_RDAP_COMMON['notices'],
        'remarks': HR_RDAP_COMMON['remarks'],
        'roles': {
            '_short': 'Roles',
            '_name': 'Roles',
            '_description': 'List of roles assigned to a registered object.'
        },
        'status': HR_RDAP_COMMON['status'],
    }
}

HR_WHOIS = {
    'nets': {
        '_short': 'Network',
        '_name': 'RIR Network',
        '_description': 'The assigned network for an IP address. May be a '
                        'parent or child network.',
        'address': {
            '_short': 'Address',
            '_name': 'Postal Address',
            '_description': 'The contact postal address.'
        },
        'cidr': {
            '_short': 'CIDR Blocks',
            '_name': 'Classless Inter-Domain Routing Blocks',
            '_description': 'Network routing blocks an IP address belongs to.'
        },
        'city': {
            '_short': 'City',
            '_name': 'City',
            '_description': 'The city registered with a whois network.'
        },
        'country': {
            '_short': 'Country Code',
            '_name': 'Country Code',
            '_description': 'Country code registered for the network in '
                            'ISO 3166-1 format.'
        },
        'created': {
            '_short': 'Created',
            '_name': 'Created Timestamp',
            '_description': 'The date the network was created in ISO 8601 '
                            'format.'
        },
        'description': {
            '_short': 'Description',
            '_name': 'Description',
            '_description': 'The description for the network.'
        },
        'emails': {
            '_short': 'Emails',
            '_name': 'Email Addresses',
            '_description': 'The contact email addresses.'
        },
        'handle': {
            '_short': 'Handle',
            '_name': 'RIR Network Handle',
            '_description': 'Unique identifier for a registered network.'
        },
        'name': {
            '_short': 'Name',
            '_name': 'RIR Network Name',
            '_description': 'The identifier assigned to the network '
                            'registration for an IP address.'
        },
        'postal_code': {
            '_short': 'Postal',
            '_name': 'Postal Code',
            '_description': 'The postal code registered with a whois network.'
        },
        'range': {
            '_short': 'Ranges',
            '_name': 'CIDR Block Ranges',
            '_description': 'Network routing blocks an IP address belongs to.'
        },
        'state': {
            '_short': 'State',
            '_name': 'State',
            '_description': 'The state registered with a whois network.'
        },
        'updated': {
            '_short': 'Updated',
            '_name': 'Updated Timestamp',
            '_description': 'The date the network was updated in ISO 8601 '
                            'format.'
        }
    },
    'referral': {
        '_short': 'Referral',
        '_name': 'Referral Whois',
        '_description': 'The referral whois data if referenced and enabled.',
    }
}

HR_WHOIS_NIR = {
    'nets': {
        '_short': 'NIR Network',
        '_name': 'National Internet Registry Network',
        '_description': 'The assigned NIR (JPNIC, KRNIC) network for an IP '
                        'address. May be a parent or child network.',
        'address': {
            '_short': 'Address',
            '_name': 'Postal Address',
            '_description': 'The network contact postal address.'
        },
        'cidr': {
            '_short': 'CIDR Blocks',
            '_name': 'Classless Inter-Domain Routing Blocks',
            '_description': 'Network routing blocks an IP address belongs to.'
        },
        'country': {
            '_short': 'Country Code',
            '_name': 'Country Code',
            '_description': 'Country code registered for the network in '
                            'ISO 3166-1 format.'
        },
        'handle': {
            '_short': 'Handle',
            '_name': 'NIR Network Handle',
            '_description': 'Unique identifier for a registered NIR network.'
        },
        'name': {
            '_short': 'Name',
            '_name': 'NIR Network Name',
            '_description': 'The identifier assigned to the network '
                            'registration for an IP address.'
        },
        'postal_code': {
            '_short': 'Postal',
            '_name': 'Postal Code',
            '_description': 'The postal code registered with a NIR network.'
        },
        'range': {
            '_short': 'Ranges',
            '_name': 'CIDR Block Ranges',
            '_description': 'Network routing blocks an IP address belongs to.'
        },
        'nameservers': {
            '_short': 'NS',
            '_name': 'Nameservers',
            '_description': 'Nameservers associated with a NIR network.'
        },
        'created': {
            '_short': 'Created',
            '_name': 'Created Timestamp',
            '_description': 'The date the network was created in ISO 8601 '
                            'format.'
        },
        'updated': {
            '_short': 'Updated',
            '_name': 'Updated Timestamp',
            '_description': 'The date the network was updated in ISO 8601 '
                            'format.'
        },
        'contacts': {
            '_short': 'Contacts',
            '_name': 'NIR Contacts',
            '_description': 'The contacts (admin, tech) registered with a NIR '
                            'network.',
            'organization': {
                '_short': 'Org',
                '_name': 'Organization',
                '_description': 'The contact organization.'
            },
            'division': {
                '_short': 'Div',
                '_name': 'Division',
                '_description': 'The contact division of the organization.'
            },
            'name': {
                '_short': 'Name',
                '_name': 'Name',
                '_description': 'The contact name.'
            },
            'title': {
                '_short': 'Title',
                '_name': 'Title',
                '_description': 'The contact position or job title.'
            },
            'phone': {
                '_short': 'Phone',
                '_name': 'Phone Number',
                '_description': 'The contact phone number.'
            },
            'fax': {
                '_short': 'Fax',
                '_name': 'Fax Number',
                '_description': 'The contact fax number.'
            },
            'email': {
                '_short': 'Email',
                '_name': 'Email Address',
                '_description': 'The contact email address.'
            },
            'reply_email': {
                '_short': 'Reply Email',
                '_name': 'Reply Email Address',
                '_description': 'The contact reply email address.'
            },
            'updated': {
                '_short': 'Updated',
                '_name': 'Updated Timestamp',
                '_description': 'The date the contact was updated in ISO 8601 '
                                'format.'
            }
        }
    }
}



#!/usr/bin/env python
#coding:utf-8
"""
  Author:   --<v1ll4n>
  Purpose: Utils Classes
  Created: 2016/12/13
"""

import unittest


########################################################################
class Singleton(object):
    """"""
    #_instance = None

    #----------------------------------------------------------------------
    def __new__(cls, *args, **kwargs):
        """singleton class wrapper"""
        if not hasattr(cls, '_instance'):
            origin = super(Singleton, cls)
            cls._instance = origin.__new__(cls, *args, **kwargs)
        return cls._instance
        
    
    
if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python
#coding:utf-8
"""
  Author:   --<v1ll4n>
  Purpose: Provide some useful thread utils
  Created: 2016/10/29
"""

import unittest
#import multiprocessing 
from pprint import pprint
from time import sleep
from queue import Full, Empty, Queue
#from random import choice
#from traceback import format_exc
from threading import Thread, Lock
#from multiprocessing import Process, Lock
from uuid import uuid1


########################################################################
class TaskError(Exception):
    """"""
    pass
        
########################################################################
class LaborThread(Thread):
    """"""
    
    #----------------------------------------------------------------------
    def __init__(self, result_queue, master, clean_mod=True, *args, **kargs):
        """Constructor"""
        Thread.__init__(self, name='ThreadPool-Labor-'+uuid1().hex,
                        *args, **kargs)
        
        self._master = master
        
        self._clean_mod = clean_mod
        
        self._result_queue = result_queue
    
        self._startworkingflag_ = True
        
        self._task_queue = Queue(1)
        
        self._count_lock = Lock()
    
    #----------------------------------------------------------------------
    def get_result_queue(self):
        """"""
        return self._result_queue
        
    #----------------------------------------------------------------------
    def get_task_queue(self):
        """"""
        return self._task_queue
    
    #----------------------------------------------------------------------
    def feed(self, function, *vargs, **kwargs):
        """"""
        try:
            self._task_queue.put_nowait(tuple([function, vargs, kwargs]))
            return True
        except Full:
            #format_exc()
            return False
    
    #----------------------------------------------------------------------
    def run(self):
        """"""
        while self._startworkingflag_:
            #pprint('Running')
            try:
                _task = self._task_queue.get(timeout=3)
                result = {}
                result['from'] = self.name
                result['state'] = False
                result['result'] = None
                result['current_task'] = _task.__str__()
                result['exception'] = tuple()
                try:
                    ret = self._process_task(_task)
                    result['state'] = True
                    result['result'] = ret
                    #self._result_queue.put(result)                    
                except Exception as e:
                    result['state'] = False
                    result['result'] = None
                    exception_i = (str(type(e)), str(e))
                    result['exception'] = exception_i
                finally:
                    if self._clean_mod:
                        _result = {}
                        _result['state'] = result['state']
                        _result['result'] = result['result']
                        result = _result
                    self._result_queue.put(result)
                
                self._count_lock.acquire()
                self._master._executed_task_count = \
                    self._master._executed_task_count + 1
                self._count_lock.release()
            except Empty:
                pass
    
    #----------------------------------------------------------------------
    def _process_task(self, task):
        """"""
        try:
            ret = task[0](*task[1], **task[2])
            return ret 
        except Exception as e:
            raise e
            
        
    #----------------------------------------------------------------------
    def stop(self):
        """"""
        #self.stop()
        self._startworkingflag_ = False
    
    #----------------------------------------------------------------------
    def __del__(self):
        """"""
        self.stop()
        
    #----------------------------------------------------------------------
    def _exception_process(self):
        """"""
        
    

########################################################################
class Pool(object):
    """"""

    #----------------------------------------------------------------------
    def __init__(self, thread_max=30, clean_mod=True):
        """Constructor"""
        self.thread_max = thread_max
        
        self._current_thread = []
        self._daemon_thread = []
        self._clean_mod = clean_mod
        self._result_queue = Queue()
        
        self._task_queue = Queue()
        
        self.is_alive = True
        
        self._executed_task_count = 0
        self._task_count = 0
        
    #----------------------------------------------------------------------
    def _restart_thread_daemon(self):
        """"""
        #pprint('threads daemon started!')
        while self.is_alive:
            if len(self._current_thread) < self.thread_max:
                self._start_new_labor()
            else:
                sleep(0.5)
    
    #----------------------------------------------------------------------
    def _start_new_labor(self):
        """"""
        #pprint('start new labor')
        _tmp_labor = LaborThread(result_queue=self._result_queue, master=self,
                                 clean_mod=self._clean_mod)
        _tmp_labor.daemon = True
        _tmp_labor.start()
        self._current_thread.append(_tmp_labor)
        
    #----------------------------------------------------------------------
    def feed(self, target_func, *vargs, **kwargs):
        """"""
        self._task_queue.put(tuple([target_func, vargs, kwargs]))
        self._task_count = self._task_count + 1
        
    #----------------------------------------------------------------------
    def _dispatcher(self):
        """"""
        #pprint('dispatcher start!')
        while self.is_alive:
            try:
                ret = self._task_queue.get()
                while True:
                    availible_threads = [None if x.get_task_queue().full() else x for x in self._current_thread]
                    for i in availible_threads:
                        if i == None:
                            pass
                        else:
                            i.feed(ret[0], *ret[1], **ret[2])
                            ret = None
                            break
                    if ret == None:
                        break
                    else:
                        continue
            except Empty:
                sleep(seconds=0.5)
        
    #----------------------------------------------------------------------
    def stop(self):
        """"""
        for i in self._current_thread:
            i.stop()
            del i
        self.is_alive = False
        
    #----------------------------------------------------------------------
    def start(self):
        """"""
        self.is_alive = True
        
        _ = Thread(name='restart_labor_daemon', target=self._restart_thread_daemon)
        _.daemon = True
        _.start()
        self._daemon_thread.append(_)
        
        _ = Thread(name='dispatcher_daemon', target=self._dispatcher)
        _.daemon = True
        _.start()

        
    #----------------------------------------------------------------------
    def get_result_queue(self):
        """"""
        return self._result_queue
    
    #----------------------------------------------------------------------
    def get_task_queue(self):
        """"""
        return self._task_queue
    
    #----------------------------------------------------------------------
    def get_result_generator(self):
        """"""
        while True:
            try:
                ret = self._result_queue.get(timeout=1)
                yield ret
            except Empty:
                if self._task_count == self._executed_task_count:
                    break
                else:
                    pass
    
    #----------------------------------------------------------------------
    @property    
    def task_count(self):
        """The amount of tasks"""
        return self._task_count
    
    #----------------------------------------------------------------------
    @property    
    def executed_task_count(self):
        """"""
        return self._executed_task_count
    
    #----------------------------------------------------------------------
    @property    
    def percent(self):
        """"""
        return float(self._executed_task_count)/float(self._task_count)
        
        
        
        
                    


########################################################################
class PoolTest(unittest.case.TestCase):
    """"""

    #----------------------------------------------------------------------
    def runTest(self):
        """Constructor"""
        self.test_laborprocess()
        
    #----------------------------------------------------------------------
    def test_pool(self):
        """"""
        def func1(arg1):
            print('func1 called!')
            return arg1
            
        pool = Pool()
        pool.start()
        pool.feed(func1, 12345)
        for i in range(10):
            pool.feed(func1, i)
        sleep(3)
        while True:
            try:
                pprint(pool.get_result_queue().get(timeout=5))
            except Empty:
                break
            
        pool.stop()
if __name__ == "__main__":
    unittest.main()


# Copyright (c) 2013, 2014, 2015, 2016 Philip Hane
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import sys
from xml.dom.minidom import parseString
from os import path
import re
import copy
import io
import csv
import logging

if sys.version_info >= (3, 3):  # pragma: no cover
    from ipaddress import (ip_address,
                           ip_network,
                           IPv4Address,
                           IPv4Network,
                           IPv6Address,
                           summarize_address_range,
                           collapse_addresses)
else:  # pragma: no cover
    from ipaddr import (IPAddress as ip_address,
                        IPNetwork as ip_network,
                        IPv4Address,
                        IPv4Network,
                        IPv6Address,
                        summarize_address_range,
                        collapse_address_list as collapse_addresses)

try:  # pragma: no cover
    from itertools import filterfalse

except ImportError:  # pragma: no cover
    from itertools import filterfalse as filterfalse

log = logging.getLogger(__name__)

IETF_RFC_REFERENCES = {
    # IPv4
    'RFC 1122, Section 3.2.1.3':
    'http://tools.ietf.org/html/rfc1122#section-3.2.1.3',
    'RFC 1918': 'http://tools.ietf.org/html/rfc1918',
    'RFC 3927': 'http://tools.ietf.org/html/rfc3927',
    'RFC 5736': 'http://tools.ietf.org/html/rfc5736',
    'RFC 5737': 'http://tools.ietf.org/html/rfc5737',
    'RFC 3068': 'http://tools.ietf.org/html/rfc3068',
    'RFC 2544': 'http://tools.ietf.org/html/rfc2544',
    'RFC 3171': 'http://tools.ietf.org/html/rfc3171',
    'RFC 919, Section 7': 'http://tools.ietf.org/html/rfc919#section-7',
    # IPv6
    'RFC 4291, Section 2.7': 'http://tools.ietf.org/html/rfc4291#section-2.7',
    'RFC 4291': 'http://tools.ietf.org/html/rfc4291',
    'RFC 4291, Section 2.5.2':
    'http://tools.ietf.org/html/rfc4291#section-2.5.2',
    'RFC 4291, Section 2.5.3':
    'http://tools.ietf.org/html/rfc4291#section-2.5.3',
    'RFC 4291, Section 2.5.6':
    'http://tools.ietf.org/html/rfc4291#section-2.5.6',
    'RFC 4291, Section 2.5.7':
    'http://tools.ietf.org/html/rfc4291#section-2.5.7',
    'RFC 4193': 'https://tools.ietf.org/html/rfc4193'
}

IP_REGEX = (
    r'(?P<ip>'
    # IPv4
    '(((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)(\.)){3}'
    '(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)'
    # IPv6
    '|\[?(((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:)'
    '{6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|'
    '2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]'
    '{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d'
    '\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|'
    '((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|'
    '2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]'
    '{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)'
    '(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(('
    '(:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1'
    '\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(('
    '[0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4})'
    '{0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]'
    '?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:(('
    '25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})'
    ')|:)))(%.+)?))\]?'
    # Optional IPv4 Port
    '((:(6553[0-5]|655[0-2]\d|65[0-4]\d{2}|6[0-4]\d{3}|[1-5]\d{4}|[1-9]\d{0,3}'
    # Optional CIDR block
    '))|(\/(?:[012]\d?|3[012]?|[4-9])))?'
    ')'
)


def ipv4_lstrip_zeros(address):
    """
    The function to strip leading zeros in each octet of an IPv4 address.

    Args:
        address: An IPv4 address in string format.

    Returns:
        String: The modified IPv4 address string.
    """

    # Split  the octets.
    obj = address.strip().split('.')

    for x, y in enumerate(obj):

        # Strip leading zeros. Split / here in case CIDR is attached.
        obj[x] = y.split('/')[0].lstrip('0')
        if obj[x] in ['', None]:
            obj[x] = '0'

    return '.'.join(obj)


def calculate_cidr(start_address, end_address):
    """
    The function to calculate a CIDR range(s) from a start and end IP address.

    Args:
        start_address: The starting IP address in string format.
        end_address: The ending IP address in string format.

    Returns:
        List: A list of calculated CIDR ranges.
    """

    tmp_addrs = []

    try:

        tmp_addrs.extend(summarize_address_range(
            ip_address(start_address),
            ip_address(end_address)))

    except (KeyError, ValueError, TypeError):  # pragma: no cover

        try:

            tmp_addrs.extend(summarize_address_range(
                ip_network(start_address).network_address,
                ip_network(end_address).network_address))

        except AttributeError:  # pragma: no cover

            tmp_addrs.extend(summarize_address_range(
                ip_network(start_address).ip,
                ip_network(end_address).ip))

    return [i.__str__() for i in collapse_addresses(tmp_addrs)]


def get_countries(is_legacy_xml=False):
    """
    The function to generate a dictionary containing ISO_3166-1 country codes
    to names.

    Args:
        is_legacy_xml: Boolean for whether to use the older country code
            list (iso_3166-1_list_en.xml).

    Returns:
        Dictionary: A dictionary with the country codes as the keys and the
            country names as the values.
    """

    # Initialize the countries dictionary.
    countries = {}

    # Set the data directory based on if the script is a frozen executable.
    if sys.platform == 'win32' and getattr(sys, 'frozen', False):

        data_dir = path.dirname(sys.executable)  # pragma: no cover

    else:

        data_dir = path.dirname(__file__)

    if is_legacy_xml:

        log.debug('Opening country code legacy XML: {0}'.format(
                str(data_dir) + '/data/iso_3166-1_list_en.xml'))

        # Create the country codes file object.
        f = io.open(str(data_dir) + '/data/iso_3166-1_list_en.xml', 'r',
                    encoding='ISO-8859-1')

        # Read the file.
        data = f.read()

        # Check if there is data.
        if not data:  # pragma: no cover

            return {}

        # Parse the data to get the DOM.
        dom = parseString(data)

        # Retrieve the country entries.
        entries = dom.getElementsByTagName('ISO_3166-1_Entry')

        # Iterate through the entries and add to the countries dictionary.
        for entry in entries:

            # Retrieve the country code and name from the DOM.
            code = entry.getElementsByTagName(
                'ISO_3166-1_Alpha-2_Code_element')[0].firstChild.data
            name = entry.getElementsByTagName(
                'ISO_3166-1_Country_name')[0].firstChild.data

            # Add to the countries dictionary.
            countries[code] = name.title()

    else:

        log.debug('Opening country code CSV: {0}'.format(
                str(data_dir) + '/data/iso_3166-1_list_en.xml'))

        # Create the country codes file object.
        f = io.open(str(data_dir) + '/data/iso_3166-1.csv', 'r',
                    encoding='utf-8')

        # Create csv reader object.
        csv_reader = csv.reader(f, delimiter=',', quotechar='"')

        # Iterate through the rows and add to the countries dictionary.
        for row in csv_reader:

            # Retrieve the country code and name columns.
            code = row[0]
            name = row[1]

            # Add to the countries dictionary.
            countries[code] = name

    return countries


def ipv4_is_defined(address):
    """
    The function for checking if an IPv4 address is defined (does not need to
    be resolved).

    Args:
        address: An IPv4 address in string format.

    Returns:
        Tuple:

        :Boolean: True if given address is defined, otherwise False
        :String: IETF assignment name if given address is defined, otherwise ''
        :String: IETF assignment RFC if given address is defined, otherwise ''
    """

    # Initialize the IP address object.
    query_ip = IPv4Address(str(address))

    # This Network
    if query_ip in IPv4Network('0.0.0.0/8'):

        return True, 'This Network', 'RFC 1122, Section 3.2.1.3'

    # Loopback
    elif query_ip.is_loopback:

        return True, 'Loopback', 'RFC 1122, Section 3.2.1.3'

    # Link Local
    elif query_ip.is_link_local:

        return True, 'Link Local', 'RFC 3927'

    # IETF Protocol Assignments
    elif query_ip in IPv4Network('192.0.0.0/24'):

        return True, 'IETF Protocol Assignments', 'RFC 5736'

    # TEST-NET-1
    elif query_ip in IPv4Network('192.0.2.0/24'):

        return True, 'TEST-NET-1', 'RFC 5737'

    # 6to4 Relay Anycast
    elif query_ip in IPv4Network('192.88.99.0/24'):

        return True, '6to4 Relay Anycast', 'RFC 3068'

    # Network Interconnect Device Benchmark Testing
    elif query_ip in IPv4Network('198.18.0.0/15'):

        return (True,
                'Network Interconnect Device Benchmark Testing',
                'RFC 2544')

    # TEST-NET-2
    elif query_ip in IPv4Network('198.51.100.0/24'):

        return True, 'TEST-NET-2', 'RFC 5737'

    # TEST-NET-3
    elif query_ip in IPv4Network('203.0.113.0/24'):

        return True, 'TEST-NET-3', 'RFC 5737'

    # Multicast
    elif query_ip.is_multicast:

        return True, 'Multicast', 'RFC 3171'

    # Limited Broadcast
    elif query_ip in IPv4Network('255.255.255.255/32'):

        return True, 'Limited Broadcast', 'RFC 919, Section 7'

    # Private-Use Networks
    elif query_ip.is_private:

        return True, 'Private-Use Networks', 'RFC 1918'

    return False, '', ''


def ipv6_is_defined(address):
    """
    The function for checking if an IPv6 address is defined (does not need to
    be resolved).

    Args:
        address: An IPv6 address in string format.

    Returns:
        Tuple:

        :Boolean: True if address is defined, otherwise False
        :String: IETF assignment name if address is defined, otherwise ''
        :String: IETF assignment RFC if address is defined, otherwise ''
    """

    # Initialize the IP address object.
    query_ip = IPv6Address(str(address))

    # Multicast
    if query_ip.is_multicast:

        return True, 'Multicast', 'RFC 4291, Section 2.7'

    # Unspecified
    elif query_ip.is_unspecified:

        return True, 'Unspecified', 'RFC 4291, Section 2.5.2'

    # Loopback.
    elif query_ip.is_loopback:

        return True, 'Loopback', 'RFC 4291, Section 2.5.3'

    # Reserved
    elif query_ip.is_reserved:

        return True, 'Reserved', 'RFC 4291'

    # Link-Local
    elif query_ip.is_link_local:

        return True, 'Link-Local', 'RFC 4291, Section 2.5.6'

    # Site-Local
    elif query_ip.is_site_local:

        return True, 'Site-Local', 'RFC 4291, Section 2.5.7'

    # Unique Local Unicast
    elif query_ip.is_private:

        return True, 'Unique Local Unicast', 'RFC 4193'

    return False, '', ''


def unique_everseen(iterable, key=None):
    """
    The generator to list unique elements, preserving the order. Remember all
    elements ever seen. This was taken from the itertools recipes.

    Args:
        iterable: An iterable to process.
        key: Optional function to run when checking elements (e.g., str.lower)

    Returns:
        Generator: Yields a generator object.
    """

    seen = set()
    seen_add = seen.add

    if key is None:

        for element in filterfalse(seen.__contains__, iterable):

            seen_add(element)
            yield element

    else:

        for element in iterable:

            k = key(element)

            if k not in seen:

                seen_add(k)
                yield element


def unique_addresses(data=None, file_path=None):
    """
    The function to search an input string and/or file, extracting and
    counting IPv4/IPv6 addresses/networks. Summarizes ports with sub-counts.
    If both a string and file_path are provided, it will process them both.

    Args:
        data: A string to process.
        file_path: An optional file path to process.

    Returns:
        Dictionary:

        :ip address/network: Each address or network found is a dictionary w/\:

            :count: Total number of times seen (Integer)
            :ports: Dictionary with port numbers as keys and the number of
                times seen for this ip as values (Dictionary)

    Raises:
        ValueError: Arguments provided are invalid.
    """

    if not data and not file_path:

        raise ValueError('No data or file path provided.')

    ret = {}
    base = {
        'count': 0,
        'ports': {}
    }

    file_data = None
    if file_path:

        log.debug('Opening file for unique address analysis: {0}'.format(
                str(file_path)))

        f = open(str(file_path), 'r')

        # Read the file.
        file_data = f.read()

    pattern = re.compile(
        str(IP_REGEX),
        re.DOTALL
    )

    # Check if there is data.
    log.debug('Analyzing input/file data'.format(
                str(file_path)))
    for input_data in [data, file_data]:

        if input_data:

            # Search for IPs.
            for match in pattern.finditer(input_data):

                is_net = False
                port = None
                try:

                    found = match.group('ip')

                    if '.' in found and ':' in found:

                        split = found.split(':')
                        ip_or_net = split[0]
                        port = split[1]

                    elif '[' in found:

                        split = found.split(']:')
                        ip_or_net = split[0][1:]
                        port = split[1]

                    elif '/' in found:

                        is_net = True
                        ip_or_net = found

                    else:

                        ip_or_net = found

                    if is_net:

                        ip_obj = ip_network(ip_or_net)

                    else:
                        ip_obj = ip_address(ip_or_net)

                    obj_str = ip_obj.__str__()

                    if obj_str not in list(ret.keys()):

                        ret[obj_str] = copy.deepcopy(base)

                    ret[obj_str]['count'] += 1

                    if port:

                        try:

                            ret[obj_str]['ports'][str(port)] += 1

                        except KeyError:

                            ret[obj_str]['ports'][str(port)] = 1

                except (KeyError, ValueError):

                    continue

    return ret



#!/usr/bin/env python
#coding:utf-8
"""
  Author:   --<v1ll4n>
  Purpose: Caculate About IP using IPy
  Created: 2016/12/26
"""

import unittest

from IPy import IP
from .ipwhois import IPWhois
from random import randint, seed
from time import time

seed(time())


#----------------------------------------------------------------------
def is_ipv4(ip):
    """Check if the [ip] is a real ip addr.
    
    Params:
        ip: :str: General IP format.
        
    Returns:
        :type: bool
        :desc: if ip is a valid IP addr, return true
            else return False"""
    try:
        IP(ip)
        return True
    except ValueError:
        return False

#----------------------------------------------------------------------
def get_all_c_class(ip):
    """Return the All C Class Addr (v4)"""
    for i in IP(ip).make_net('255.255.255.0'):
        yield i.strNormal()
    
#----------------------------------------------------------------------
def get_ip_block_from_ipwhois(ip):
    """"""
    if is_ipv4(ip):
        the_whole_ip_block = IPWhois(ip).lookup_whois()['asn_cidr']
    else:
        raise ValueError('Plz Input A valid ip addr(v4)!')
    
    for i in IP(the_whole_ip_block):
        yield i.strNormal()

#----------------------------------------------------------------------
def ip2int(ip):
    """"""
    return IP(ip).int()

#----------------------------------------------------------------------
def ipv4_range(start_ip, end_ip):
    """"""
    start = ip2int(start_ip)
    end = ip2int(end_ip)
    
    for i in range(start, end):
        yield IP(i).strNormal()
    
#----------------------------------------------------------------------
def random_ip(start='0.0.0.0', end='224.0.0.0'):
    """"""
    return IP(randint(IP(start).int(), IP(end).int())).strNormal()
    

if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python
#coding:utf-8
"""
  Author:   --<v1ll4n>
  Purpose: Import Utils
  Created: 2017/1/2
"""

import unittest
import os
import sys

#----------------------------------------------------------------------
def import_by_path(path, module_name, basedir='.'):
    """"""
    dirname = os.path.dirname(basedir)
    absdir = os.path.abspath(dirname)
    absdir = os.path.join(absdir, path)
    sys.path.append(absdir)
    mod = __import__(module_name)
    sys.path.remove(absdir)
    
    return mod

if __name__ == '__main__':
    unittest.main()